// ****************************************************************************************
//
//
// This code generates all the frames for a 10-second animated png
//  which demonstrates a long move.
// 
// 
// The first second is static, before. The last second is static after.
//
//  During the animation there are 4 moving objects.
//    A black peg enters from offscreen right.
//    A black link3 enters from offscreen left.
//    Two links on the board are rotated and repositioned.
//
// 
// ImageMagick will buid the animated png file.


#version 3.7;




// For the static frames,
// To avoid generating the same frame multiple times, we use a post-frame command.
// This command is in povray.ini in this directory.


#fopen Fix_id "fix.sh" write   // This bash script file will either be blank,
                               // or will contain a cp command to (usually) copy a source image
                               // (which is a static frame 
                               // of this POVray script which was already generated))
                               // into this frame in a post-frame command.
                               // The file which gets clobbered should be an all black frame.
                               // If this frame is the first of a static interval,
                               // it gets copied into /images/static.

#include "ap_skip.inc"        // Reference an array, using clock, to see which of three cases:
                               //   1. This is an animated interval
                               //   2. This is the first frame of a POV static interval
                               //   3. This static POV frame has already been generated


#if ( DoThis )
                           // One of the first two cases is true, so generate a framwe.

  #if (Animating)
                           // The first case is true. This frame is part of an animation.
  
    #fclose Fix_id          // Whatever fix.sh contained from the last frame, it is blank now.
                            // We won't need to copy this frame into ./static_frames,
                            // and we don't want this frame to be destroyed by a post-frame command. 
                            //
    
    #warning concat( "***** ANIM Time= ", str(ThisTime, 7, 4), "\n")
  
  #else
  
    #warning concat( "********* FIRST Time= ", str(ThisTime, 7, 4), "\n")
    #write (Fix_id, CpCommand)
    #fclose Fix_id
  
  #end

  // The #else block, where DoThis is false, is at the end of this code.
    
    
  #include "colors.inc"
  #include "woods.inc"
  #include "glass.inc"
  #include "textures.inc"
  #include "chars.inc"
  #include "debug.inc"
  #include "finish.inc"
  #include "metals.inc"
  #include "stones1.inc"
//  #include "crystal.ttf"


  
  #include "ap_constants.inc"
  #include "ap_objects.inc"
//  #include "ap_games.inc"
//  #include "ap_instructions.inc"
      
    

#global_settings
{ assumed_gamma 1
}

// camera { location <-5, 25, -20>*separation*1.3 look_at <9*separation, 0, 7*separation> angle 15 }


// increase camera height  from 6
camera { location <8.5, 7, 7>*separation look_at <8.5*separation, 0, 8.5*separation> angle 50 }  


light_source

{ <8, 30, -12>*separation , 1
  area_light x*50,y*50,15,15 circular orient jitter adaptive 0
}



  
//---------------------- The far wall
// object {Wall}


//----------------------- THE TABLE
// object {Table}

  
  
  
  
//------------------------ GAME POSITION

  
  
// This places the static elements of the apng.  
  
union
{ object { TwixtBoard }

  #declare PegCoords = WhitePegs;
  #declare CurPeg = WhitePeg;
  #declare Link1Coords = WhiteLink1s;
  #declare Link2Coords = WhiteLink2s;
  #declare Link3Coords = WhiteLink3s;
  #declare Link4Coords = WhiteLink4s;
  #declare CurLink = WhiteLink;
  #declare Cnt = 0;                 // This code assumes there will always be more elements in the
  #while(Cnt < 2)                   // WhitePegs and/or BlackPegs arrays than in any of the Link arrays.
    #declare PegXPos = 1;
    #declare Link1XPos = 1;
    #declare Link2XPos = 1;
    #declare Link3YPos = 1;
    #declare Link4YPos = 1;
    #declare Ind = 0;
    #while(Ind < dimension_size(PegCoords, 1))
      #if(PegCoords[Ind] = 0) #declare PegXPos = PegXPos+1;
      #else
        object { CurPeg translate PegPos(PegXPos, PegCoords[Ind]) }
      #end
      #if (Ind < dimension_size(Link1Coords, 1))
        #if (Link1Coords[Ind] = 0) #declare Link1XPos = Link1XPos+1;
        #else
          object { CurLink rotate y*Theta translate  Link12Pos(Link1XPos, Link1Coords[Ind]) }
        #end
      #end
      #if (Ind < dimension_size(Link2Coords, 1))
        #if (Link2Coords[Ind] = 0) #declare Link2XPos = Link2XPos+1;
        #else
          object { CurLink rotate y*(-Theta) translate  Link12Pos(Link2XPos, Link2Coords[Ind]) }
        #end
      #end
      #if (Ind < dimension_size(Link3Coords, 1))
        #if (Link3Coords[Ind] = 0) #declare Link3YPos = Link3YPos+1;
        #else
          object { CurLink rotate y*(Theta-90) translate  Link34Pos(Link3Coords[Ind], Link3YPos) }
        #end
      #end
      #if (Ind < dimension_size(Link4Coords, 1))
        #if (Link4Coords[Ind] = 0) #declare Link4YPos = Link4YPos+1;
        #else
          object { CurLink rotate y*(-Theta-90) translate  Link34Pos(Link4Coords[Ind], Link4YPos) }
        #end
      #end
      #declare Ind = Ind+1;
    #end
    #declare PegCoords = BlackPegs;
    #declare CurPeg = BlackPeg;
    #declare Link1Coords = BlackLink1s;
    #declare Link2Coords = BlackLink2s;
    #declare Link3Coords = BlackLink3s;
    #declare Link4Coords = BlackLink4s;
    #declare CurLink = BlackLink;
    #declare Cnt = Cnt+1;
  #end
}

#include "ap_animate.inc"   // place the animated objects.
  
#else   // In this case, we are in the middle of a static interval.
        // There is no frame to generate.
  #debug concat( "********* STATIC Time= ", str(ThisTime, 7, 4), "\n")
  #write (Fix_id, CpCommand)  // This post frame command will copy an already generated image
                               // into the blank frame that this POVray iteration generates.
  #fclose Fix_id
  
#end
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  