// *******************************************************************************************************
//
// This defines some "universal" constants, and values specific to this animated png.

#declare s5 = sqrt(5);

// Maybe Grid Size will change over the course of a video?

#declare GS = 24;            // Grid Size, should be divisible by 6. Likely 18, 24, or 30.
#declare HGS = GS/2;         // Half of grid size
#declare HGSM1 = HGS-1;      // 11 for standard grid, 14 for size 30
#declare HGSM15 = HGS-1.5;   // 10.5 for standard grid. Used for placing border lines and guide lines.
#declare GSM3D3 = (GS-3)/3;  // Used for "two diamonds" pattern.


// Each object definition is assigned an identifier constant with a unique value.
// The main file T01_Rules.pov will scan through the instances, looking for those that pertain to
// the current clock time. Then it runs the identifier through a long case conditional
// to determine which objects need to be instantiated.

#declare BOARDID = 0;    // An informal upper limit of 10 characters is placed on these IDs.
#declare TABLEID = 1;    // This is for the sake of how the code in the instructions.inc file
#declare W_PEGID  = 2;   // will look.
#declare B_PEGID  = 3;
#declare W_LINKID = 4;  
#declare B_LINKID = 5;  
#declare W_PEG3MID = 6;
#declare W_LINK3MID = 7; 
#declare B_PEG3MID = 8;
#declare B_LINK3MID = 9; 



//     7       0
//      \     /
// 6__   \   /   __1
//    --__\ /__--
//      ___*___               This is how linking directions are numbered.
//  __--  / \  --__           
// 5     /   \     2          
//      /     \               Before they are rotated around the Y axis, 
//     4       3              links point "up" (between 7 and 0.)
//                            This is a reference to the constant array ROT_ANGLE,
//                            which defines what angle each of these directions corresponds to.
// 
// Keep in mind the trigonometric functions define positive angles as counterclockwise,
// whereas the rotate command in POVray goes clockwise around the y axis from this perspective.


#declare ATH = degrees(atan(0.5));
#declare AT2 = degrees(atan(2));
#declare HPI = 90;                 // Half of PI radians, in degrees (This is ridiculous BTW.)

#declare ROT_ANGLE = array[8] {ATH, AT2, ATH+HPI, AT2+HPI, -AT2-HPI, -ATH-HPI, -AT2, -ATH }





// ********************************************************************************************************
//
// This defines all the constants needed for display of the flat set.
// 



// #declare LettersF = "ABCDEFGHiJKLMNOPQRSTUVWX"

// #declare NumbersF = " 1 2 3 4 5 6 7 8 9101112131415161718192021222324" // no coordinates for this apng

#declare separation = 2/3;                // 2/3 of an inch is the standard width of each column
                                          // or row. Most of the object constants are based on this.
#declare epsilon = 0.001;

#declare PegHeight = 0.3*separation;

#declare PegOuter = 0.3*separation;        // PegOutF + (BorderWidth/2) must be <= separation/2

#declare PegInner = 0.25*separation;

#declare HoleRad = 0.15*separation;

#declare HoleDepth = 0.2*separation;
                                           // For version 2, the links are redesigned
                                           // to make them easier to pick up and to rest stably
                                           // without wobbling. 
                                           // 
#declare LinkWidth = 0.15*separation;

#declare LinkRad = LinkWidth/2;

#declare LinkGap =0.05*separation;       // horizontal gap between underside of link and  peg

                                          // LinkToCenter is horizontal distance from where the underside
                                          // of a link clears past the inner wall of peg, to peg center.
                                          // the link extends beyond that point, but that part is a box
                                          // which extends down into the peg cavity.
                                          // We have the relationship
                                          // (PegInF)^2 = (LinkToCntrF)^2 + (LinkRadF)^2

#declare LinkToCenter = sqrt(PegInner*PegInner - LinkRad*LinkRad);

#declare LinkLength = s5*separation - 2*(LinkToCenter - LinkRad);    // length of upper portion of link

#declare HLL = LinkLength/2;                                         // half the link length

#declare LinkLengthUnder = s5*separation - 2*(PegOuter+LinkGap);

#declare HLU = LinkLengthUnder/2;

                                      // LinkRiserRadius is the radius of the cylinder which 
                                      // generates the riser above the link for players to grab.
                                      // Given the riser has max height of LinkRadF above the surface of
                                      // the link at the link midpoint,
                                      // which tapers down to flush with the link surface at either end,
                                      // a bit of algebra results in the following formula.

#declare LinkRiserRadius =  (HLL*HLL - LinkRad*LinkRad) / (2*LinkRad);

                           // LinkRiserCenter is the y value of center of the cylinder that generates the
                           // riser for players to grab.

#declare LinkRiserCenter = (PegHeight + LinkWidth) - LinkRiserRadius;

#declare LinkRiserThickness = LinkRad;
                                                   // Trim is unique to the flat set, needs no final F
#declare TrimE = LinkRad - (LinkRiserThickness/2); // The algebra behind these values is complicated
                                                   // and needs a diagram. See Trim_values.png
#declare TrimC = (LinkRad*LinkRad)/TrimE;        // The purpose is to deternine the minimum dimensions of
                                                       // a box which will be rotated and used to trim away
#declare TrimA = sqrt(TrimC*TrimC + LinkRad*LinkRad);  // the corners of each link so that they do not
                                                       // intersect with another link tip on the same peg.
#declare TrimB = (TrimE*TrimC)/TrimA;

#declare TrimAlpha = degrees(atan2(LinkRad, TrimC));


#declare LetterStart = 0-(11.5*separation);

#declare NumberStart = 11.5*separation;

#declare BorderWidth = 0.15*separation;

#declare LineWidth = 0.06 * separation;

#declare Theta = degrees(atan2(1,2));


//**************************************************************************************************
//
// Wall , table not needed for apng long move
//

// The following arrays define the location of the static pegs and links.

#declare WhitePegs = array[30]
{
/* A */  0,
/* B */  0,
/* C */  0,
/* D */  0,
/* E */  0,
/* F */  0,
/* G */  0,
/* H */  0,
/* i */  0,
/* J */  0,
/* K */  0,
/* L */  0,
/* M */  0,
/* N */  0,
/* O */  0,
/* P */  0,
/* Q */  0,
/* R */  5,0,
/* S */  0,
/* T */  5,0,
/* U */  6,0
/* V */  2,3,0,
/* W */  5,0,
/* X */  0
 
};

#declare BlackPegs = array[30]
{
/* A */  0,
/* B */  0,
/* C */  0,
/* D */  0,
/* E */  0,
/* F */  0,
/* G */  0,
/* H */  0,
/* i */  0,
/* J */  0,
/* K */  0,
/* L */  0,
/* M */  0,
/* N */  0,
/* O */  0,
/* P */  0,
/* Q */  0,
/* R */  0,
/* S */  0,
/* T */  3,4,0,
/* U */  2,5,0,
/* V */  5,0,   // after, there is a black peg on 4.
/* W */  3,0,
/* X */  0
 
};


#declare BlackLink1s = array[24]  // Link1 is nne-ssw relative to the board.
{                                 // (For example, the white borders are on the n and s 
                                  // sides of the board.)
/* AB */  0,
/* BC */  0,
/* CD */  0,
/* DE */  0,
/* EF */  0,
/* FG */  0,
/* GH */  0,
/* Hi */  0,
/* iJ */  0,
/* JK */  0,
/* KL */  0,
/* LM */  0,
/* MN */  0,
/* NO */  0,
/* OP */  0,
/* PQ */  0,
/* QR */  0,
/* RS */  0,
/* ST */  0,
/* TU */  6,0,  // T'3 is animated later, so it is placed within ap_animate.inc
/* UV */  0,
/* VW */  0,
/* WX */  0

};

#declare BlackLink2s = array[23]             // Link2 is nnw-sse
{
/* AB */  0,
/* BC */  0,
/* CD */  0,
/* DE */  0,
/* EF */  0,
/* FG */  0,
/* GH */  0,
/* Hi */  0,
/* iJ */  0,
/* JK */  0,
/* KL */  0,
/* LM */  0,
/* MN */  0,
/* NO */  0,
/* OP */  0,
/* PQ */  0,
/* QR */  0,
/* RS */  0,
/* ST */  0,
/* TU */  0,  // T'4 link2 after
/* UV */  0,  // U'3 link2 after
/* VW */  0,
/* WX */  0

};

#declare BlackLink3s = array[24]              // Link3 is wnw-ese
{
/* 1-2 */   0,
/* 2-3 */   22,0,
/* 3-4 */   0,     // 21 not listed because it will be animated.
/* 4-5 */   0,     // 21 not listed because it becomes animated.
/* 5-6 */   0,
/* 6-7 */   0,
/* 7-8 */   0,
/* 8-9 */   0,
/* 9-10 */  0,
/* 10-11 */ 0,
/* 11-12 */ 0,
/* 12-13 */ 0,
/* 13-14 */ 0,
/* 14-15 */ 0,
/* 15-16 */ 0,
/* 16-17 */ 0,
/* 17-18 */ 0,
/* 18-19 */ 0,
/* 19-20 */ 0,
/* 20-21 */ 0,
/* 21-22 */ 0,
/* 22-23 */ 0,
/* 23-24 */ 0

};

#declare BlackLink4s = array[23]              // Link4 is wsw-ene
{
/* 1-2 */     0,
/* 2-3 */     0,
/* 3-4 */     0,
/* 4-5 */     0,
/* 5-6 */     0,
/* 6-7 */     0,
/* 7-8 */     0,
/* 8-9 */     0,
/* 9-10 */    0,
/* 10-11 */   0,
/* 11-12 */   0,
/* 12-13 */   0,
/* 13-14 */   0,
/* 14-15 */   0,
/* 15-16 */   0,
/* 16-17 */   0,
/* 17-18 */   0,
/* 18-19 */   0,
/* 19-20 */   0,
/* 20-21 */   0,
/* 21-22 */   0,
/* 22-23 */   0,
/* 23-24 */   0

};

#declare WhiteLink1s = array[25]
{
/* AB */      0,
/* BC */      0,
/* CD */      0,
/* DE */      0,
/* EF */      0,
/* FG */      0,
/* GH */      0,
/* Hi */      0,
/* iJ */      0,
/* JK */      0,
/* KL */      0,
/* LM */      0,
/* MN */      0,
/* NO */      0,
/* OP */      0,
/* PQ */      0,
/* QR */      0,
/* RS */      0,
/* ST */      6,0,
/* TU */      0,
/* UV */      0,
/* VW */      6,0,
/* WX */      0

};

#declare WhiteLink2s = array[26]
{
/* AB */     0,
/* BC */     0,
/* CD */     0,
/* DE */     0,
/* EF */     0,
/* FG */     0,
/* GH */     0,
/* Hi */     0,
/* iJ */     0,
/* JK */     0,
/* KL */     0,
/* LM */     0,
/* MN */     0,
/* NO */     0,
/* OP */     0,
/* PQ */     0,
/* QR */     4,0,
/* RS */     6,0,
/* ST */     0,
/* TU */     0,
/* UV */     0,
/* VW */     4,0,
/* WX */     0

};

#declare WhiteLink3s = array[23]
{
/* 1-2 */    0,
/* 2-3 */    0,
/* 3-4 */    0,
/* 4-5 */    0,
/* 5-6 */    0,
/* 6-7 */    0,
/* 7-8 */    0,
/* 8-9 */    0,
/* 9-10 */   0,
/* 10-11 */  0,
/* 11-12 */  0,
/* 12-13 */  0,
/* 13-14 */  0,
/* 14-15 */  0,
/* 15-16 */  0,
/* 16-17 */  0,
/* 17-18 */  0,
/* 18-19 */  0,
/* 19-20 */  0,
/* 20-21 */  0,
/* 21-22 */  0,
/* 22-23 */  0,
/* 23-24 */  0

};

#declare WhiteLink4s = array[24]
{
/* 1-2 */     0,
/* 2-3 */     0,
/* 3-4 */     0,
/* 4-5 */     0,
/* 5-6 */     22,0,
/* 6-7 */     0,
/* 7-8 */     0,
/* 8-9 */     0,
/* 9-10 */    0,
/* 10-11 */   0,
/* 11-12 */   0,
/* 12-13 */   0,
/* 13-14 */   0,
/* 14-15 */   0,
/* 15-16 */   0,
/* 16-17 */   0,
/* 17-18 */   0,
/* 18-19 */   0,
/* 19-20 */   0,
/* 20-21 */   0,
/* 21-22 */   0,
/* 22-23 */   0,
/* 23-24 */   0

};


























